.module       MIDI;

.include <system.k>;
.include <samp.k>;

.global         process_midi;
.global         inputmidi;

.global         hexprint;
.external       nSend;

{ do this simply }
.var    midi_vel,               { MIDI velocity }
        midi_note,              { MIDI note }
        midi_cont,              { MIDI controller number }
        midi_val;               { MIDI controller value }

{ variables for the MIDI input state machine }
.var    inputmidi,              { MIDI input data from serial port }
        midi_status,midi_ch,    { MIDI Status, MIDI channel }
        midi_state,             { MIDI state machine }
        midi_data1,midi_data2,  { incoming midi data }
        midi_onnote;            { which note is actually being played ? }

.init midi_state:  0x0;
.init midi_status: 0x0;

.var    midi_play_note[16];
.init   midi_play_note[0]:0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff;
.init   midi_play_note[8]:0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff;

{ let the cartooooooooons, begin! -------------------------------------- }
process_midi:   

        { read incoming byte }
        i4 = dm (PTR_TO_GET_CHAR);call (i4);
        if lt jump end_proc_midi; { time out }


        dm(inputmidi) = ax1;

        call nSend;  { echo it }

        ax1=dm(inputmidi);
        ar = tstbit 7 of ax1;
        if ne jump MIDIstatus;

MIDIdata:
        { shift MIDI data in }
        ax0 = dm(midi_data2);
        dm(midi_data1) = ax0;
        dm(midi_data2) = ax1;

        ar  = dm(midi_state);
        none = pass ar;
        if ne jump MIDIevent;

        ax1 = 1;                { wait for another byte }
        dm(midi_state)=ax1;
        jump end_proc_midi;


MIDIevent:
        ax1 = 0;                  { initialize input data buffer }
        dm(midi_state)=ax1;

        ax1 = dm(midi_status);
        ay0 = 0xf0;     { mask off channel number; leave message type }
        ar  = ax1 and ay0;
        ax1 = ar;

        { look for MIDI note on messages }
        ay0 = 0x0090;{ note on =1001xxxx} none = ax1 - ay0;if eq jump MIDInoteon;

        { look for MIDI note off messages }
        ay0 = 0x0080;{ note on =1000xxxx} none = ax1 - ay0;if eq jump MIDInoteoff;

        { look for MIDI Controller messages on any channel }
        ay0 = 0x00b0;{ control =1011xxxx} none = ax1 - ay0;if eq jump MIDIcontr;

        { look for Pitch Wheel messages on any channel }
        ay0 = 0x00e0;{ wheel =1110xxxx} none = ax1 - ay0;if eq jump MIDIwheel;


        { ignore other types of messages (for now!) }
        jump end_proc_midi;

MIDIstatus:
        { ax1 = dm(inputmidi); }
        ay0 = 0xf0;
        ar  = ax1 and ay0;              
        ar  = ar - ay0;
        if eq jump MIDIrealtime;

        dm(midi_status)=ax1;      { set this as our new status }

        ax0=0x0;
        dm(midi_state)=ax0;       { initialize input data buffer }

        ay1 = dm(midi_status);    { compute midi channel number }
        ax1 = 0x000f;
        ar  = ax1 and ay1;
        dm(midi_ch) = ar;               

        jump end_proc_midi;


{ f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,fa,fb,fc,fd,fe,ff }
MIDIrealtime:
        { don't do anything with these yet }

        ax1 = 0;                  { initialize input data buffer }
        dm(midi_state)=ax1;

        jump end_proc_midi;

{ handle MIDI Note on message --------------------------------------- }
MIDInoteon:         
        ay0=dm(midi_ch);
        ax1=0x0000;none=ax1-ay0;if eq jump synth_noteon;
        ax1=0x0001;none=ax1-ay0;if eq jump synth_noteon;
        ax1=0x0002;none=ax1-ay0;if eq jump synth_noteon;
        ax1=0x0003;none=ax1-ay0;if eq jump synth_noteon;
        jump end_proc_midi;     { ignore other channels }

synth_noteon:
        ay1 = dm(midi_data1);           { get midi_note number }
        ar  = ^midi_play_note;          { store playing note }
        ar  = ar + ay0;                 { ay0 is midi ch }
        i0  = ar;
        l0  = 0;
        dm(i0,m0) = ay1;                { store midi_note number }

        ax1 = dm(midi_data2);           { get midi_vel value }

        call newsound;          { update wave_delta according to new note }

        jump end_proc_midi;

{ handle MIDI Note Off message --------------------------------------- }
MIDInoteoff:
        ay0=dm(midi_ch);
        ax1=0x0000;none=ax1-ay0;if eq jump synth_noteoff;
        ax1=0x0001;none=ax1-ay0;if eq jump synth_noteoff;
        ax1=0x0002;none=ax1-ay0;if eq jump synth_noteoff;
        ax1=0x0003;none=ax1-ay0;if eq jump synth_noteoff;
        jump end_proc_midi;     { ignore other channels }

synth_noteoff:
        ax1 = dm(midi_data1);           { get midi_note number }
        ar  = ^midi_play_note;          { store playing note }
        ar  = ar + ay0;                 { ay0 is midi ch }
        i0  = ar;
        l0  = 0;
        m0  = 0;
        ay1 = dm(i0,m0);                { get current note on that chan }

        { only kill if this note is one this channel }
        none= ax1 - ay1; if ne jump end_proc_midi;
        dm(i0,m0) = 0x80;               { nothing playing now }

        call killsound;                 { update envelope status }

        jump end_proc_midi;

{ handle MIDI Control message ------------------------------------ }
MIDIcontr:

        ay0 = dm(midi_data1);           { get midi_cont number }
        dm(midi_cont) = ay0;

        ax0 = dm(midi_data2);           { get midi_val value }
        dm(midi_val) = ax0;

        { jump to appropriate routine for controller type }
        ax1 = dm(midi_cont);
        call synth_cont;

        jump end_proc_midi;


{ handle MIDI Pitch Wheel message --------------------------------------- }
MIDIwheel:         
        ax1 = dm(midi_data1);           { get upper byte of wheel }
        ax1 = dm(midi_data1);           { get lower byte of wheel }

        jump end_proc_midi;

end_proc_midi:
        rts;

{ put number to be printed in ax1 }
{ uses ar, sr, ax0, ay0, ay1, etc }
hexprint:
        ay0 = ax1;
        ar  = ay0;sr  = lshift ar by -12 (lo);   call hexdigit;call nSend;
        ar  = ay0;sr  = lshift ar by -8 (lo);    call hexdigit;call nSend;
        ar  = ay0;sr  = lshift ar by -4 (lo);    call hexdigit;call nSend;
        ar  = ay0;sr  = lshift ar by 0 (lo);     call hexdigit;call nSend;
        ax1 = 0x20;call nSend;
        rts;

hexdigit:
        ay1 = 0x000f;
        ax0 = sr0;
        ar  = ax0 and ay1;      { separate the digit we want }

        ax0 = ar;
        ay1 = 0x0a;
        ar  = ax0 - ay1;
        if ge jump hex_letter;

hex_number:
        ay1 = 0x003a;
        ar  = ar + ay1;
        ax1 = ar;
        rts;

hex_letter:
        ay1 = 0x41;
        ar  = ar + ay1;
        ax1 = ar;
        rts;



.endmod;

